/*
Example for AnyNet_2G Click

    Date          : Nov 2017.
    Author        : MikroE Team

Test configuration STM32 :
    
    MCU              : STM32F107VCT6
    Dev. Board       : EasyMx PRO v7 for STM32
    ARM Compiler ver : v5.1.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes all necessary GPIO pins, UART used for
the communcation with GSM module and UART used for infromation logging
- Application Initialization - Initializes driver, power on module and sends few
command for the default module configuration
- Application Task - running in parallel core state machine and api state 
machine.

Additional Functions :

All additional functions such as timer initialization and various handlers 
are defined inside API module provided in form of source code.

Notes :

- Echo must be turned off for succefull parsing. ( ATE0 )
- UART polling works much better with HFC enabled.
- In case of usage of other MCUs Timer initialization must be adjusted according to your MCU.

*/

#include "Click_AnyNet_2G_types.h"
#include "Click_AnyNet_2G_config.h"
#include "__anynet2g_api.h"

static uint8_t getCS()
{
    return GPIOD_IDR.B13;;
}

static void setRST( uint8_t state )
{
    GPIOC_ODR.B2 = state;
}

static void setPWM( uint8_t state )
{
    GPIOA_ODR.B0 = state;
}

static void setINT( uint8_t state )
{
    GPIOD_ODR.B10 = 0;
}

const T_hal_gpioObj _MIKROBUS_GPIO = 
{
    { 0, setRST, 0, 0, 0, 0, setPWM, setINT, 0, 0, 0, 0 },
    { 0, 0, getCS, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

const T_hal_uartObj _MIKROBUS_UART = 
{
    UART3_Write,
    UART3_Read,
    UART3_Data_Ready
};

void systemInit()
{
    GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_13);
    GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_2 );
    GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_0 );
    GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_10);

    UART1_Init_Advanced( 115200, _UART_8_BIT_DATA,
                                 _UART_NOPARITY,
                                 _UART_ONE_STOPBIT,
                                 &_GPIO_MODULE_USART1_PA9_10 );

    UART3_Init_Advanced( 9600, _UART_8_BIT_DATA,
                               _UART_NOPARITY,
                               _UART_ONE_STOPBIT,
                               &_GPIO_MODULE_USART3_PD89 );
    Delay_ms( 500 );
}


void applicationInit()
{
// API INIT (TIMER & FLAGS)
    api_anynet2g_init();

// DRIVER AND CORE INIT
    anynet2g_uartDriverInit((T_ANYNET2G_P)&_MIKROBUS_GPIO, (T_ANYNET2G_P)&_MIKROBUS_UART);
    anynet2g_coreInit( api_anynet2g_default_handler, 1500 );

// HANDLERS ASSIGNATION
    anynet2g_setHandler( "+AWSPUBOPEN", 1500, api_anynet2g_openTopicAWS_handler );
    anynet2g_setHandler( "+AWSSUBOPEN", 1500, api_anynet2g_openSubscribeAWS_handler );
    anynet2g_setHandler( "+AWSPUBCLOSE", 1500, api_anynet2g_closeTopicAWS_handler );
    anynet2g_setHandler( "+AWSSTATE", 1500, api_anynet2g_checkStateAWS_handler );

// ENABLE HFC - HANDSHAKE
    anynet2g_hfcEnable( true );
    anynet2g_modulePower( true );

// MODULE INIT
    anynet2g_cmdSingle( "AT" );
    api_anynet2g_firmwareID();
    api_anynet2g_queryICCID();

    api_anynet2g_checkStateAWS( _ANYNET2G_GET );

// OPEN TOPIC 0 AND PUBLISH
    api_anynet2g_openTopicAWS( _ANYNET2G_SET, "0", "MY_TOPIC" );
    api_anynet2g_publishTopicAWS( _ANYNET2G_SET, "0", "MIKROE", "" );
    
// CLOSE TOPIC
    //api_anynet2g_closeTopicAWS( _ANYNET2G_SET, "0" );

// SUBSCRIBE
    //api_anynet2g_openSubscribeAWS( _ANYNET2G_SET, "0", "MYTOP1" );
}

void applicationTask()
{
// CORE STATE MACHINE
    anynet2g_process();
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}